@@ -124,7 +124,7 @@
 #' | [`"energy"`][method_energy] | Energy balancing |
 #'
 #' `method` can also be supplied as a user-defined function; see
-#' [`method_user`] for instructions and examples.
+#' [`method_user`] for instructions and examples. Setting `method = NULL` computes unit weights.
 #'
 #' When using `weightit()`, please cite both the \pkg{WeightIt} package
 #' (using `citation("WeightIt")`) and the paper(s) in the references
@@ -139,13 +139,12 @@
 #' matrix of covariates and a vector of treatment statuses rather than a
 #' formula and data frame and performs minimal argument checking and
 #' processing. It may be useful for speeding up simulation studies for which
-#' the correct arguments are known. In general `weightit()` should be
+#' the correct arguments are known. In general, `weightit()` should be
 #' used.
 #'
 #' [summary.weightit()] for summarizing the weights
 #'
 #' @examples
-#'
 #' library("cobalt")
 #' data("lalonde", package = "cobalt")
 #'
@@ -194,6 +193,7 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
   if (is_null(treat)) {
     .err("no treatment variable was specified")
   }
+
   if (length(treat) != nrow(covs)) {
     .err("the treatment and covariates must have the same number of units")
   }
@@ -201,7 +201,7 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
   n <- length(treat)
 
   if (anyNA(treat)) {
-    .err("no missing values are allowed in the treatment variable")
+    .err("missing values are not allowed in the treatment variable")
   }
 
   #Get treat type
@@ -214,7 +214,9 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
 
   #Process ps
   ps <- .process_ps(ps, data, treat)
-  if (is_not_null(ps) && !identical(method, "glm") && !is.function(method)) {
+  if (is_not_null(ps) && is_not_null(method) &&
+      !is.function(method) &&
+      !identical(method, "glm")) {
     .wrn("`ps` is supplied, so `method` will be ignored")
     method <- "glm"
   }
@@ -222,11 +224,15 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
   ##Process method
   .check_acceptable_method(method, msm = FALSE, force = FALSE)
 
-  if (is.character(method)) {
+  if (is_null(method)) {
+    method <- NULL
+  }
+  else if (is.character(method)) {
     method <- .method_to_proper_method(method)
+    .check_method_treat.type(method, treat.type)
     attr(method, "name") <- method
   }
-  else if (is.function(method)) {
+  else { #function
     method.name <- deparse1(substitute(method))
     .check_user_method(method)
     attr(method, "name") <- method.name
@@ -242,20 +248,21 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
 
   #Process missing
   missing <- {
-    if (!anyNA(reported.covs)) ""
-    else .process_missing(missing, method, treat.type)
+    if (is_null(method) || !anyNA(reported.covs)) ""
+    else .process_missing(missing, method)
   }
 
   #Check subclass
   if (is_not_null(subclass)) .check_subclass(method, treat.type)
 
   #Process s.weights
-  s.weights <- process.s.weights(s.weights, data)
+  s.weights <- .process.s.weights(s.weights, data)
 
   if (is_null(s.weights)) s.weights <- rep.int(1, n)
+  else .check_method_s.weights(method, s.weights)
 
   #Process stabilize
-  if (isFALSE(stabilize)) {
+  if (is_null(method) || isFALSE(stabilize)) {
     stabilize <- NULL
   }
   else if (isTRUE(stabilize)) {
@@ -263,15 +270,22 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
   }
 
   if (is_not_null(stabilize)) {
-    if (treat.type != "continuous" && estimand != "ATE") {
-      .err("`stabilize` can only be supplied when `estimand = \"ATE\"`")
+    if (is.character(method) && !.weightit_methods[[method]]$stabilize_ok) {
+      .wrn(sprintf("`stabilize` cannot be used with %s and will be ignored",
+                   .method_to_phrase(method)))
+      stabilize <- NULL
     }
+    else {
+      if (treat.type != "continuous" && estimand != "ATE") {
+        .err("`stabilize` can only be supplied when `estimand = \"ATE\"`")
+      }
 
-    if (!rlang::is_formula(stabilize)) {
-      .err("`stabilize` must be `TRUE`, `FALSE`, or a formula with the stabilization factors on the right hand side")
+      if (!rlang::is_formula(stabilize)) {
+        .err("`stabilize` must be `TRUE`, `FALSE`, or a formula with the stabilization factors on the right hand side")
+      }
+
+      stabilize <- update(formula, update(stabilize, NULL ~ .))
     }
-
-    stabilize <- update(formula, update(stabilize, NULL ~ .))
   }
 
   ##Process by
@@ -286,52 +300,57 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
   processed.by <- .process_by(by, data = data, treat = treat, by.arg = by.arg)
 
   #Process moments and int
-  moments.int <- .process_moments_int(moments, int, method)
+  m.i.q <- .process_moments_int_quantile(moments = moments,
+                                         int = int,
+                                         quantile = A[["quantile"]],
+                                         method = method)
 
   call <- match.call()
   # args <- list(...)
 
   ## Running models ----
 
-  #Returns weights (weights) and propensity score (ps)
-  A[["treat"]] <- treat
-  A[["covs"]] <- covs
-  A[["s.weights"]] <- s.weights
-  A[["by.factor"]] <- attr(processed.by, "by.factor")
-  A[["estimand"]] <- estimand
-  A[["focal"]] <- focal
-  A[["stabilize"]] <- FALSE
-  A[["method"]] <- method
-  A[["moments"]] <- moments.int[["moments"]]
-  A[["int"]] <- moments.int[["int"]]
-  A[["subclass"]] <- subclass
-  A[["ps"]] <- ps
-  A[["missing"]] <- missing
-  A[["verbose"]] <- verbose
-  A[["include.obj"]] <- include.obj
-  A[[".data"]] <- data
-  A[[".formula"]] <- formula
-  A[[".covs"]] <- reported.covs
+  A["treat"] <- list(treat)
+  A["covs"] <- list(covs)
+  A["s.weights"] <- list(s.weights)
+  A["by.factor"] <- list(attr(processed.by, "by.factor"))
+  A["estimand"] <- list(estimand)
+  A["focal"] <- list(focal)
+  A["stabilize"] <- list(FALSE)
+  A["method"] <- list(method)
+  A[c("moments", "int", "quantile")] <- m.i.q[c("moments", "int", "quantile")]
+  A["subclass"] <- list(subclass)
+  A["ps"] <- list(ps)
+  A["missing"] <- list(missing)
+  A["verbose"] <- list(verbose)
+  A["include.obj"] <- list(include.obj)
+  A[".data"] <- list(data)
+  A[".formula"] <- list(formula)
+  A[".covs"] <- list(reported.covs)
 
+  #Returns weights (weights) and propensity score (ps)
   obj <- do.call("weightit.fit", A)
 
   if (is_not_null(stabilize)) {
     stab.t.c <- get_covs_and_treat_from_formula(stabilize, data)
 
-    A[["treat"]] <- stab.t.c[["treat"]]
-    A[["covs"]] <- stab.t.c[["model.covs"]]
-    A[["method"]] <- "glm"
-    A[["moments"]] <- NULL
-    A[["int"]] <- NULL
-    A[[".formula"]] <- stabilize
-    A[[".covs"]] <- stab.t.c[["reported.covs"]]
+    A["treat"] <- list(stab.t.c[["treat"]])
+    A["covs"] <- list(stab.t.c[["model.covs"]])
+    A["method"] <- list("glm")
+    A["moments"] <- list(integer())
+    A["int"] <- list(FALSE)
+    A["quantile"] <- list(list())
+    A[".formula"] <- list(stabilize)
+    A[".covs"] <- list(stab.t.c[["reported.covs"]])
 
     sw_obj <- do.call("weightit.fit", A)
 
     obj$weights <- obj$weights / sw_obj[["weights"]]
   }
 
-  .check_estimated_weights(obj$weights, treat, treat.type, s.weights)
+  if (is_not_null(method) && is_null(ps)) {
+    .check_estimated_weights(obj$weights, treat, treat.type, s.weights)
+  }
 
   ## Assemble output object----
   out <- list(weights = obj$weights,
@@ -372,13 +391,19 @@ weightit <- function(formula, data = NULL, method = "glm", estimand = "ATE", sta
 print.weightit <- function(x, ...) {
   treat.type <- get_treat_type(x[["treat"]])
 
-  cat("A " %+% italic(class(x)[1]) %+% " object\n")
+  cat(sprintf("A %s object\n", italic(class(x)[1])))
 
   if (is_not_null(x[["method"]])) {
     cat(sprintf(" - method: %s (%s)\n",
                 add_quotes(attr(x[["method"]], "name")),
                 .method_to_phrase(x[["method"]])))
   }
+  else if (all_the_same(x[["weights"]])) {
+    cat(" - method: no weighting\n")
+  }
+  else if (is_not_null(x[["ps"]])) {
+    cat(" - method: propensity score weighting\n")
+  }
 
   cat(sprintf(" - number of obs.: %s\n",
               length(x[["weights"]])))
@@ -389,9 +414,9 @@ print.weightit <- function(x, ...) {
   cat(sprintf(" - treatment: %s\n",
               switch(treat.type,
                      "continuous" = "continuous",
-                     "multinomial" = sprintf("%s-category (%s)",
-                                             nunique(x[["treat"]]),
-                                             paste(levels(x[["treat"]]), collapse = ", ")),
+                     "multi-category" = sprintf("%s-category (%s)",
+                                       nunique(x[["treat"]]),
+                                       word_list(levels(x[["treat"]]), and.or = FALSE)),
                      "binary" = "2-category")))
 
   if (is_not_null(x[["estimand"]])) {
@@ -403,17 +428,17 @@ print.weightit <- function(x, ...) {
   if (is_not_null(x[["covs"]])) {
     cat(sprintf(" - covariates: %s\n",
                 if (length(names(x[["covs"]])) > 60) "too many to name"
-                else paste(names(x[["covs"]]), collapse = ", ")))
+                else word_list(names(x[["covs"]]), and.or = FALSE)))
   }
 
   if (is_not_null(x[["by"]])) {
-    cat(sprintf(" - by: %s\n", paste(names(x[["by"]]), collapse = ", ")))
+    cat(sprintf(" - by: %s\n", word_list(names(x[["by"]]), and.or = FALSE)))
   }
 
   if (is_not_null(x[["moderator"]])) {
     nsubgroups <- nlevels(attr(x[["moderator"]], "by.factor"))
     cat(sprintf(" - moderator: %s (%s subgroups)\n",
-                paste(names(x[["moderator"]]), collapse = ", "),
+                word_list(names(x[["moderator"]]), and.or = FALSE),
                 nsubgroups))
   }
 
