@@ -1,4 +1,9 @@
 test_that("No weights", {
+  skip_if_not_installed("sandwich")
+  skip_if_not_installed("brglm2")
+
+  eps <- if (capabilities("long.double")) 1e-5 else 1e-1
+
   test_data <- readRDS(test_path("fixtures", "test_data.rds"))
 
   expect_no_condition({
@@ -19,7 +24,7 @@ test_that("No weights", {
                data = test_data, family = binomial)
   expect_equal(coef(fit0), coef(fit_g))
   expect_equal(vcov(fit0), sandwich::sandwich(fit_g),
-               tolerance = 1e-6)
+               tolerance = eps)
 
   #Offset
   set.seed(123)
@@ -36,7 +41,7 @@ test_that("No weights", {
                data = test_data, family = binomial)
   expect_equal(coef(fit), coef(fit_g))
   expect_equal(vcov(fit), sandwich::sandwich(fit_g),
-               tolerance = 1e-6)
+               tolerance = eps)
 
   #Cluster-robust SEs
   clus <- sample(1:50, nrow(test_data), replace = TRUE)
@@ -52,7 +57,7 @@ test_that("No weights", {
                data = test_data, family = binomial)
   expect_equal(coef(fit), coef(fit_g))
   expect_equal(vcov(fit), sandwich::vcovCL(fit_g, cluster = clus),
-               tolerance = 1e-5)
+               tolerance = eps)
 
   #BR
   expect_no_condition({
@@ -69,6 +74,10 @@ test_that("No weights", {
 })
 
 test_that("Binary treatment", {
+  skip_if_not_installed("fwb")
+
+  eps <- if (capabilities("long.double")) 1e-5 else 1e-1
+
   test_data <- readRDS(test_path("fixtures", "test_data.rds"))
 
   expect_no_condition({
@@ -77,8 +86,6 @@ test_that("Binary treatment", {
                    include.obj = TRUE)
   })
 
-  expect_M_parts_okay(W)
-
   expect_no_condition({
     fit0 <- glm_weightit(Y_C ~ A * (X1 + X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9),
                         data = test_data, weightit = W)
